// DemoPanel.java
//
// (C) Copyright IBM Corporation. 2006, 2006. All Rights Reserved.
//
/*
 * Created on Feb 15, 2006
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package ibm.rational.pplus.demo.form;

import ibm.rational.pplus.demo.wordcount.WordCount;

import java.awt.BorderLayout;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Timer;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.table.JTableHeader;

import Rational.PureAPI;

public class DemoPanel extends JPanel {
	// Private fields
	private WordCount 		wordCount = new WordCount();
	private FileInputStream inputStream;
	private long			inputLength;
	private ParsingThread 	parsingThread;
	private Timer			timer;
	private DemoTimerTask	timerTask;
	
	private long			timeStart;
	private long			timeEnd;
	private long			memoryStart;
	private long			memoryEnd;
	
	private boolean 		isRunning = false;

	// UI components
	private JScrollPane jScrollPaneResults = null;
	private JTable jTableResults = null;
	private JPanel jPanelControl = null;
	private JPanel jPanelFile = null;
	private JPanel jPanelQaConsole = null;
	private JToolBar jToolBarStatus = null;
	private JTextField jTextFieldFile = null;
	private JButton jButtonFile = null;
	private JButton jButtonRun = null;
	private JCheckBox jCheckBoxPureApi = null;
	private JLabel jLabelWords = null;
	private JTextField jTextFieldWords = null;
	private JLabel jLabelBytes = null;
	private JTextField jTextFieldBytes = null;
	private JProgressBar jProgressBar = null;
	private ResultsTableModel resultsTableModel = null;
	private JLabel jLabelMemoryUsed = null;
	private JTextField jTextFieldMemoryUsed = null;
	private JLabel jLabelElapsedTime = null;
	private JTextField jTextFieldElapsedTime = null;
	private JButton jButtonUntested = null;
	/**
	 * This is the default constructor
	 */
	public DemoPanel() {
		super();
		initialize();
	}

	/*
	 * 
	 */
	public String getFileName() {
		return this.jTextFieldFile.getText();
	}

	/*
	 * 
	 */
	public InputStream getInputStream()
	{
		return inputStream;
	}

	/*
	 * 
	 */
	private void runFileDialog()
	{
		Frame f = new Frame();
		FileDialog fd = new FileDialog(f);
		fd.setModal(true);
		fd.setVisible(true);
		String FilePath = fd.getDirectory() + fd.getFile();
		jTextFieldFile.setText(FilePath);
	}
	
	public void showErrorMessage(String caption, String message)
	{
		MessageBox msg = new MessageBox(caption, message);
		msg.setVisible(true);
	}
	
	/*
	 * 
	 */
	private void startParser() {
		// Reset the progress indicator
		jProgressBar.setValue(0);
		// Cleanup the results table 
		this.getTableModel().clearData();
		// Create and run the parser thread
		if(isRunning)
		{  // Stop requested
			wordCount.stopProcessing();
			return;
		}
		try {
			File inputFile = new File(getFileName());
			inputLength = inputFile.length();
			inputStream = new FileInputStream(inputFile);
		} catch (FileNotFoundException e) {
			showErrorMessage("File open error", e.getMessage());
			return;
		}
		try
		{
			setRunning(true);
			timer = new Timer();
			timerTask = new DemoTimerTask(this, wordCount);
			timer.schedule(timerTask, 200, 200);
			parsingThread = new ParsingThread(this, wordCount);
			if(jCheckBoxPureApi.isSelected())
			{	// If PureAPI is enabled
				PureAPI.AddAnnotation("PurifyPlus Demo for Java");
				PureAPI.SaveData();		// Take a snapshot
				PureAPI.ClearData();	// Clear all counters
			}
			timeStart = System.currentTimeMillis();
			memoryStart = getHeapSize();
			parsingThread.start();
		}
		catch (Exception e)
		{
			showErrorMessage("Parsing Error", e.getMessage());
			this.notifyParseDone(null);
			return;
		}
	}

	/*
	 * Update information on timer notifications
	 */
	public void updateControls(WordCount wordCount)
	{
		int progress;
		progress = (int)((double)wordCount.GetnBytes() / inputLength * 100.);
		jTextFieldBytes.setText(String.valueOf(wordCount.GetnBytes()));
		jTextFieldWords.setText(String.valueOf(wordCount.GetnWords()));
		jProgressBar.setValue(progress); 
	}

	/*
	 * Called by the parsing thread to notify when the parsing is done
	 */
	public void notifyParseDone(WordCount wordCount) {
		timeEnd = System.currentTimeMillis();
		memoryEnd = getHeapSize();
		timerTask.cancel();
		jProgressBar.setValue(100);
		if(jCheckBoxPureApi.isSelected())
		{	// If PureAPI is enabled - take a snapshot here.
			PureAPI.SaveData();
		}
		// Time span calculation (there must be a better way)
		long time = timeEnd - timeStart;
		String memory = String.valueOf(memoryEnd);
		SimpleDateFormat sdf = new SimpleDateFormat("mm:ss.SSS");
		String strTime = sdf.format(new Date(time));
		jTextFieldElapsedTime.setText(strTime);
		jTextFieldMemoryUsed.setText(memory);
		if(wordCount != null)
		{
			jTextFieldWords.setText(String.valueOf(wordCount.GetnWords()));
			jTextFieldBytes.setText(String.valueOf(wordCount.GetnBytes()));
			this.getTableModel().setData(wordCount.m_Words);
		}
		setRunning(false);
	}
	
	/*
	 * Set the state to running / not running 
	 */
	private void setRunning(boolean running) {
		isRunning = running;
		if(isRunning)
		{
			jButtonRun.setText("STOP");
		}
		else
		{
			jButtonRun.setText("RUN");
		}
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setLayout(new BorderLayout());
		this.setSize(540, 373);
		this.add(getJPanelControl(), java.awt.BorderLayout.SOUTH);
		this.add(getJScrollPaneResults(), java.awt.BorderLayout.CENTER);
		
	}

	/**
	 * This method initializes jScrollPaneResults	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPaneResults() {
		if (jScrollPaneResults == null) {
			jScrollPaneResults = new JScrollPane();
			jScrollPaneResults.setViewportView(getJTableResults());
		}
		return jScrollPaneResults;
	}

	/**
	 * This method initializes jTableResults	
	 * 	
	 * @return javax.swing.JTable	
	 */
	private JTable getJTableResults() {
		if (jTableResults == null) {
			jTableResults = new JTable();
			jTableResults.setToolTipText("Results");
			jTableResults.setRowSelectionAllowed(false);
			final JTableHeader header = new JTableHeader(jTableResults.getColumnModel());
			header.addMouseListener(new MouseAdapter(){
				public void mouseClicked(MouseEvent e) {
					int column = jTableResults.getColumnModel().getColumnIndexAtX(e.getX());
					int sortField = getTableModel().getSortField();
					int sortOrder = getTableModel().getSortOrder();
					if(column == sortField)
					{
						sortOrder = sortOrder == 0 ? 1 : 0;
					}
					else
					{
						sortField = sortField == 0 ? 1 : 0;
					}
					getTableModel().setSort(sortOrder, sortField);
					getTableModel().sort();
				}
			});
			header.setVisible(true);
			jTableResults.setTableHeader(header);
			jTableResults.setModel(getTableModel());
		}
		return jTableResults;
	}

	/**
	 * This method initializes jPanelControl	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelControl() {
		if (jPanelControl == null) {
			jPanelControl = new JPanel();
			jPanelControl.setLayout(new BorderLayout());
			jPanelControl.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));
			jPanelControl.add(getJPanelFile(), java.awt.BorderLayout.NORTH);
			jPanelControl.add(getJPanelQaConsole(), java.awt.BorderLayout.CENTER);
			jPanelControl.add(getJToolBarStatus(), java.awt.BorderLayout.SOUTH);
		}
		return jPanelControl;
	}

	/**
	 * This method initializes jPanelFile	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelFile() {
		if (jPanelFile == null) {
			jPanelFile = new JPanel();
			jPanelFile.setLayout(new BoxLayout(getJPanelFile(), BoxLayout.X_AXIS));
			jPanelFile.add(getJTextFieldFile(), null);
			jPanelFile.add(getJButtonFile(), null);
			jPanelFile.add(getJButtonRun(), null);
		}
		return jPanelFile;
	}

	/**
	 * This method initializes jPanelQaConsole	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelQaConsole() {
		if (jPanelQaConsole == null) {
			jLabelElapsedTime = new JLabel();
			jLabelElapsedTime.setText("Elapsed Time");
			jLabelElapsedTime.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
			jLabelMemoryUsed = new JLabel();
			jLabelMemoryUsed.setText("MemoryUsed");
			jLabelMemoryUsed.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);

			GridLayout gridLayout = new GridLayout();
			gridLayout.setRows(3);
			gridLayout.setColumns(2);
			jPanelQaConsole = new JPanel();
			jPanelQaConsole.setLayout(gridLayout);
			jPanelQaConsole.add(getJCheckBoxPureApi(), null);
			jPanelQaConsole.add(getJButtonUntested(), null);
			jPanelQaConsole.add(jLabelMemoryUsed, null);
			jPanelQaConsole.add(getJTextFieldMemoryUsed(), null);
			jPanelQaConsole.add(jLabelElapsedTime, null);
			jPanelQaConsole.add(getJTextFieldElapsedTime(), null);
		}
		return jPanelQaConsole;
	}

	/**
	 * This method initializes jToolBarStatus	
	 * 	
	 * @return javax.swing.JToolBar	
	 */
	private JToolBar getJToolBarStatus() {
		if (jToolBarStatus == null) {
			jLabelBytes = new JLabel();
			jLabelBytes.setText("Bytes");
			jLabelWords = new JLabel();
			jLabelWords.setText("Words");
			jToolBarStatus = new JToolBar();
			jToolBarStatus.setFloatable(false);
			jToolBarStatus.add(jLabelWords);
			jToolBarStatus.add(getJTextFieldWords());
			jToolBarStatus.add(jLabelBytes);
			jToolBarStatus.add(getJTextFieldBytes());
			jToolBarStatus.add(getJProgressBar());
		}
		return jToolBarStatus;
	}

	/**
	 * This method initializes jTextFieldFile	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJTextFieldFile() {
		if (jTextFieldFile == null) {
			jTextFieldFile = new JTextField();
		}
		return jTextFieldFile;
	}

	/**
	 * This method initializes jButtonFile	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonFile() {
		if (jButtonFile == null) {
			jButtonFile = new JButton();
			jButtonFile.setText("...");
			jButtonFile.setToolTipText("Select a file for parsing");
			jButtonFile.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseClicked(java.awt.event.MouseEvent e) {
					runFileDialog();
				}
			});
			jButtonFile.addKeyListener(new java.awt.event.KeyAdapter() {
				public void keyTyped(java.awt.event.KeyEvent e) {
					runFileDialog();
				}
			});
		}
		return jButtonFile;
	}

	/**
	 * This method initializes jButtonRun	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonRun() {
		if (jButtonRun == null) {
			jButtonRun = new JButton();
			jButtonRun.setText("RUN");
			jButtonRun.setToolTipText("Start Parsing");
			jButtonRun.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseClicked(java.awt.event.MouseEvent e) {
					startParser();
				}
			});
			jButtonRun.addKeyListener(new java.awt.event.KeyAdapter() {
				public void keyTyped(java.awt.event.KeyEvent e) {
					startParser();
				}
			});
		}
		return jButtonRun;
	}

	/**
	 * This method initializes jCheckBoxPureApi	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getJCheckBoxPureApi() {
		if (jCheckBoxPureApi == null) {
			jCheckBoxPureApi = new JCheckBox();
			jCheckBoxPureApi.setText("Use PureAPI");
			jCheckBoxPureApi.setSelected(false);
			jCheckBoxPureApi.setToolTipText("PureAPI is only allowed when the sample runs with PurifyPlus");
			jCheckBoxPureApi.setEnabled(false);
			try
			{
				if(PureAPI.IsRunning() != 0)
				{
					jCheckBoxPureApi.setEnabled(true);
				}
				else
				{
					jCheckBoxPureApi.setEnabled(false);
				}
			}
			catch(NoClassDefFoundError ex)
			{
				jCheckBoxPureApi.setEnabled(false);
			}
		}
		return jCheckBoxPureApi;
	}

	/**
	 * This method initializes jButtonUntested	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonUntested() {
		if (jButtonUntested == null) {
			jButtonUntested = new JButton();
			jButtonUntested.setText("Untested Code");
			jButtonUntested.setToolTipText("Do not click.");
			jButtonUntested.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseClicked(java.awt.event.MouseEvent e) {
					untested();
				}
			});
		}
		return jButtonUntested;
	}

	private void untested()
	{
		// Untested code example:
		// This is what happens when a real product is 
		// shipped with insufficient testing.
		showErrorMessage("Warning", "Untested code. Make sure you test your code before release.");
		
	}

	/**
	 * This method initializes jTextFieldWords	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJTextFieldWords() {
		if (jTextFieldWords == null) {
			jTextFieldWords = new JTextField();
			jTextFieldWords.setEditable(false);
		}
		return jTextFieldWords;
	}

	/**
	 * This method initializes jTextFieldBytes	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJTextFieldBytes() {
		if (jTextFieldBytes == null) {
			jTextFieldBytes = new JTextField();
			jTextFieldBytes.setEditable(false);
		}
		return jTextFieldBytes;
	}

	/**
	 * This method initializes jProgressBar	
	 * 	
	 * @return javax.swing.JProgressBar	
	 */
	private JProgressBar getJProgressBar() {
		if (jProgressBar == null) {
			jProgressBar = new JProgressBar();
		}
		return jProgressBar;
	}

	/**
	 * This method initializes defaultTableModel	
	 * 	
	 * @return javax.swing.table.DefaultTableModel	
	 */
	private ResultsTableModel getTableModel() {
		if (resultsTableModel == null) {
			resultsTableModel = new ResultsTableModel();
		}
		return resultsTableModel;
	}

	/**
	 * This method initializes jTextFieldMemoryUsed	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJTextFieldMemoryUsed() {
		if (jTextFieldMemoryUsed == null) {
			jTextFieldMemoryUsed = new JTextField();
			jTextFieldMemoryUsed.setEditable(false);
		}
		return jTextFieldMemoryUsed;
	}

	/**
	 * This method initializes jTextFieldElapsedTime	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJTextFieldElapsedTime() {
		if (jTextFieldElapsedTime == null) {
			jTextFieldElapsedTime = new JTextField();
			jTextFieldElapsedTime.setEditable(false);
		}
		return jTextFieldElapsedTime;
	}

	private long getHeapSize() {
		// TODO Auto-generated method stub
		Runtime rt = Runtime.getRuntime();
		rt.gc();
		return rt.totalMemory() - rt.freeMemory();
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
