// ResultsTableModel.java
//
// (C) Copyright IBM Corporation. 2006, 2006. All Rights Reserved.
//
/*
 * Created on Feb 20, 2006
 * 
 * Table model.
 * Handles the result data presentation and sorting
 * 
 */
package ibm.rational.pplus.demo.form;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Hashtable;
import javax.swing.table.DefaultTableModel;

import ibm.rational.pplus.demo.wordcount.*;

public class ResultsTableModel extends DefaultTableModel {
	public static final int SORT_ORDER_ACCENDED  = 0;
	public static final int SORT_ORDER_DESCENDED = 1;
	public static final int SORT_FIELD_STRING    = 0;
	public static final int SORT_FIELD_FREQUENCY = 1;
	
	private ArrayList  dataCollection = new ArrayList();
	private Comparator comparator = new DataComparator();
	private int sortOrder = 0;
	private int sortField = 0;
	
	public class DataEntry implements Comparable
	{
		private String value;
		private int    frequency;
		
		public DataEntry(String value, int frequency)
		{
			this.value = value;
			this.frequency = frequency;
		}
		
		public String getValue()
		{
			return value;
		}
		
		public int getFrequency()
		{
			return frequency;
		}

		public int compareTo(Object o) {
			int result = 0;
			DataEntry d = (DataEntry)o;
			switch(sortField)
			{
			case SORT_FIELD_STRING:
				result = value.compareTo(d.getValue());
				break;
			case SORT_FIELD_FREQUENCY:
				if(frequency == d.getFrequency())
				{
					result = value.compareTo(d.getValue());
				}
				else
				{
					result = frequency > d.getFrequency() ? 1 : -1;
				}
				break;
			}
			return result;
		}
	}
	
	public class DataComparator implements Comparator
	{

		public int compare(Object o1, Object o2) {
			DataEntry e1 = (DataEntry)o1;
			DataEntry e2 = (DataEntry)o2;
			int result = e1.compareTo(e2);
			if(sortOrder == SORT_ORDER_DESCENDED)
			{
				result = -result;
			}
			return result;
		}
		
	}
	
	public ResultsTableModel() {
		super();
		addColumn("Value");
		addColumn("Frequency");
	}

	public int getRowCount() {
		int rowCount = 0;
		if(dataCollection != null)
		{
			rowCount = dataCollection.size();
		}
		return rowCount;
	}

	public int getColumnCount() {
		return 2;
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		Object val = null;
		switch(columnIndex)
		{
		case 0:
			val = ((DataEntry)dataCollection.get(rowIndex)).getValue();
			break;
		case 1:
			val = String.valueOf(((DataEntry)dataCollection.get(rowIndex)).getFrequency());
			break;
		}	
		return val;
	}

	public boolean isCellEditable(int row, int column) {
		// This table is not editable
		return false;
	}

	public void clearData() {
		dataCollection.clear();
		this.fireTableDataChanged();
	}
	
	public void setData(Hashtable data)
	{
		dataCollection.clear();
		Enumeration keys = data.keys();
		while(keys.hasMoreElements())
		{
			String key = (String)keys.nextElement();
			int value = ((WordContainer.IntBox)data.get(key)).get();
			DataEntry next = new DataEntry(key, value);
			dataCollection.add(next);
		}
		sort();
	}
	
	public void sort()
	{
		Collections.sort(dataCollection, comparator);
		this.fireTableDataChanged();
	}
	
	public void setSort(int sortOrder, int sortField)
	{
		this.sortOrder = sortOrder;
		this.sortField = sortField;
	}
	
	public int getSortOrder()
	{
		return sortOrder;
	}
	
	public int getSortField()
	{
		return sortField;
	}

}
