// WordCount.java
//
// (C) Copyright IBM Corporation. 2006, 2006. All Rights Reserved.
//
/*
 * Created on Feb 16, 2006
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package ibm.rational.pplus.demo.wordcount;

import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;

public class WordCount {

	private boolean stopProcessing = false;
	// The parser plumbing. DFA matrix etc.
 
	/// <summary>
	/// Character class. 
	/// We classify any input character to fit into one of the
	/// character classes below. This makes the DFA matrix much smaller
	/// </summary>
	public static final class CharClass_t
	{
		public static final int charEof     = 0; 			/// <summary>End of file</summary>
		public static final int charSpace   = 1;			/// <summary>White space</summary>
		public static final int charDelim   = 2;			/// <summary>Delimiter</summary>
		public static final int charValid   = 3;			/// <summary>Valid character</summary>
		public static final int charInvalid = 4;			/// <summary>Invalid character</summary>
		public static final int charLast    = 5;			    
	}

	/// <summary>
	/// Scan status.
	/// This status is used to swich DFA into a new state based on the
	/// previous status and the next input.
	/// </summary>
	public static final class ScanStat_t
	{
		public static final int statStart   = 0;			///<summary>DFA entry point</summary>
		public static final int statSpace   = 1;			///<summary>Inside WS</summary>
		public static final int statWord    = 2;			///<summary>Inside Word</summary>
		public static final int statEow     = 3;			///<summary>End of word reached</summary>
		public static final int statEof     = 4;			///<summary>End of file reached</summary>
		public static final int statError   = 5;			///<summary>Error</summary>
		public static final int statLast    = 6;				
	}
	
	int States[][] =
	{
	//				  charEof              charSpace             charDelim             charValid             charInvalid            
	/* statStart */	{ScanStat_t.statError, ScanStat_t.statSpace, ScanStat_t.statEow,   ScanStat_t.statWord,  ScanStat_t.statError},
	/* statSpace */	{ScanStat_t.statEof  , ScanStat_t.statSpace, ScanStat_t.statSpace, ScanStat_t.statWord,  ScanStat_t.statError},  
	/* statWord  */	{ScanStat_t.statEof  , ScanStat_t.statEow,   ScanStat_t.statEow,   ScanStat_t.statWord,  ScanStat_t.statError},  
	/* StatEow 	 */ {ScanStat_t.statEof  , ScanStat_t.statSpace, ScanStat_t.statEow,   ScanStat_t.statWord,  ScanStat_t.statError},  
	/* StatEof 	 */ {ScanStat_t.statError, ScanStat_t.statError, ScanStat_t.statError, ScanStat_t.statError, ScanStat_t.statError},  
	/* StatError */ {ScanStat_t.statError, ScanStat_t.statError, ScanStat_t.statError, ScanStat_t.statError, ScanStat_t.statError},  
	};								  

	//----------------------------------------------------------------------
	/// <summary>
	/// Default constructor
	/// </summary>
	public WordCount()
	{
		statNext = ScanStat_t.statStart;
		m_Words = new WordContainer();
	}

	//----------------------------------------------------------------------
	/// <summary>
	/// <c>WordCount.Parse</c>
	/// Preconditions:
	///  i_stream not null and opened
	///
	/// This method is parsing the input stream into tokens and populates
	/// the word container with the value-frequency pairs.
	/// </summary>
	/// <param name="i_stream"> Input stream representing the source to parse</param>
	public void Parse(InputStream i_stream) throws WordCountException
	{
		stopProcessing = false;
		m_stream = i_stream;
		Token tokenNext;
		m_nWords = 0;
		m_nBytes = 0;
		statNext = ScanStat_t.statStart;
		// Parsing			
		tokenNext = Scan();
		while(tokenNext.GetToken() != Token.Token_t.tokEof && ! stopProcessing)
		{
			switch(tokenNext.GetToken())
			{
				case Token.Token_t.tokWord:
					String strVal = tokenNext.GetValue();
					if(strVal.length() > 0)
					{	// Ignore empty strings;
						m_nWords++;
						m_Words.Set(strVal);
					}
					break;
				default:
					throw new WordCountException(tokenNext.GetValue());
			}
			tokenNext = Scan();
		}
	}

	//----------------------------------------------------------------------
	/// <summary>
	/// 
	/// <c>WordCount.Classify</c>
	///
	/// Classify character.
	/// input character is converted into character class (CharClass_t)
	/// Note that in this implementation char codes abov 128 are interpreted 
	/// as delimiters rather than error. This makes it possible to parse any file
	/// including binaries. This behaviour may be changed by returning
	/// CharClass_t.charError for certain characters.
	/// <seealso cref="CharClass_t"/>
	/// </summary>
	/// <param name="i_ch" type="int"> [in] character to classify </param>
	/// <returns></returns>
	/// 
	//
	private int Classify(int i_ch)
	{
		char ch = (char)i_ch;
		int ccRet;
		ccRet = CharClass_t.charInvalid;		// Assume the worst
		if(i_ch == 0xFFFF)						// End of file
			ccRet = CharClass_t.charEof;	
		else if((ch == ' ' || ch == '\t')		// White space
			 || (ch == '\r' || ch == '\n'))
			ccRet = CharClass_t.charSpace;
		else if(ch == '_')						// Undescore is part of a word
			ccRet = CharClass_t.charValid;	
		else if((ch >= 'A' && ch <= 'Z')		// Vlaid chars
    		 ||	(ch >= 'a' && ch <= 'z')
			 || (ch >= '0' && ch <= '9')
			)
			ccRet = CharClass_t.charValid;	
		else if((ch > ' ' && ch < '0')			// Delimiters
			 || (ch >= ':' && ch <= '@')
			 || (ch > 'Z' && ch < 'a')
			 || (ch > 'z' && ch <= 128)
             )
			ccRet = CharClass_t.charDelim;	
		else if(i_ch < ' ' || i_ch > 127)		// Make it configurable ?
			ccRet = CharClass_t.charDelim;		// May be considered error  or delim.
		return ccRet;
	}

	//----------------------------------------------------------------------
	/// <summary>
	/// WordCount.Scan
	/// Scan a next token from the input stream 
	/// </summary>
	/// <returns> Token - the next token parsed from the input stream </returns>
	///
	String strTok = new String();					// Using string here is a bad idea
	StringBuffer sbTok = new StringBuffer();		// StringBuilder is for mutable strings

	private Token Scan() throws WordCountException
	{
		char b = ' ';								// Next byte from the in stream
		int tokVal = Token.Token_t.tokError;
		int charNext = CharClass_t.charLast;
		strTok = "";								// Reset token value
		sbTok.delete(0, sbTok.length());
		boolean bDone = false;						// Asume we are not done yet

			while(!bDone)
			{
				// Get one char
				try
				{
					b = (char)m_stream.read();
				}
				catch(EOFException ex)
				{
					b = 0xFF;
				}
				catch(IOException ex)
				{
					throw new WordCountException("File I/O error");
				}

				m_nBytes++;
				charNext = Classify(b);
				// Fall through the DFA
				statNext = States[(int)statNext][(int)charNext];
				// Process the state
				switch(statNext)
				{
					// Space. Ignore
					case ScanStat_t.statSpace:
						break;			
					// Inside word. Keep processing
					case ScanStat_t.statWord:
						strTok += b;
						break;
					// End of word reached
					case ScanStat_t.statEow:
						tokVal = Token.Token_t.tokWord;
						bDone = true;
						break;
					// End of file reached
					case ScanStat_t.statEof:
						strTok = "<<EOF>>";
						tokVal = Token.Token_t.tokEof;
						bDone = true;
						m_nBytes--;		//Don't count eof
						break;
					// statStart is invalid state so it's processed as error
					case ScanStat_t.statError:
					case ScanStat_t.statStart:
						strTok = "<<" + b + ">>";
						tokVal = Token.Token_t.tokError;
						bDone = true;
						break;
				}
			}
		// return the next token
		return new Token(tokVal, strTok);	
	}

	//----------------------------------------------------------------------
	/// <summary>
	/// WordCount.GetnWords
	/// 
	/// Get number of words currently parsed.
	/// </summary>
	/// <returns> ulong - number of words processed </returns>
	//
	public long GetnWords()
	{
		return m_nWords;
	}

	//----------------------------------------------------------------------
	/// <summary>
	/// WordCount.GetnBytes 
	/// </summary>
	/// <returns>ulong - number of bytes processed</returns>
	//
	public long GetnBytes()
	{
		return m_nBytes;
	}
	
	//----------------------------------------------------------------------
	/// <summary>
	/// WordCount.GetnBytes 
	/// </summary>
	/// <returns>ulong - number of bytes processed</returns>
	//
	public void stopProcessing()
	{
		stopProcessing = true;
	}

	//+---------------------------------------------------------------------
	private InputStream m_stream;		// Input stream 
	private long m_nWords = 0;			// Number of words currently processed
	private long m_nBytes = 0;			// Number of bytes currently processed
	private int statNext;				// Scanner status (DFA input)
	/// <summary>List/cache of words processed</summary>
	public WordContainer m_Words;		// List of the processed words with counts
}

