////////////////////////////////////////////////////////////////////////////////
// FasterActX ActiveX Demonstration Program For Quantify
//
// | Licensed Materials - Use restricted, please refer to the "Samples" terms
// | and conditions in the IBM International Program License Agreement (IPLA).  
// (C) Copyright IBM Corporation. 1992, 2006. All Rights Reserved.
//
// Authors: Alan S. Cooper, Sean Echevarria
////////////////////////////////////////////////////////////////////////////////
// Typical Disclaimer: This FAQ is copyright (c) 1997 by Tom Armstrong
// (tom@widgetware.com). You have the right to copy and distribute this file 
// in any way you see fit as long as this paragraph is included with the 
// distribution. No warranties or claims are made as to the validity of the 
// information contained herein. So use it at your own risk.
////////////////////////////////////////////////////////////////////////////////
//
// PagePropEx.cpp : implementation file

#include "stdafx.h"
#include <afxcmn.h>			// for class CTabCtrl
#include "FasterActX.h"		// for class CMyFasterActXApp
#include "PagePropEx.h"		// for class CMyPropertyPageEx

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMyPropertyPageEx property page

IMPLEMENT_DYNCREATE(CMyPropertyPageEx, CPropertyPage)

BEGIN_MESSAGE_MAP(CMyPropertyPageEx, CPropertyPage)
	//{{AFX_MSG_MAP(CMyPropertyPageEx)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CMyPropertyPageEx::CMyPropertyPageEx(UINT nIDTemplate) 
				: CPropertyPage(nIDTemplate)
{
	m_pSheet = NULL;
	ASSERT_VALID(this);
}

CMyPropertyPageEx::CMyPropertyPageEx(CPropertySheet* pSheet, 
							   const UINT nIDTemplate, 
							   UINT nIDCaption) 
				: CPropertyPage(nIDTemplate, nIDCaption)
{
	ASSERT_VALID(pSheet);
	m_pSheet = pSheet;
	ASSERT_VALID(this);
}

CMyPropertyPageEx::CMyPropertyPageEx(CPropertySheet* pSheet, 
							   const UINT nIDTemplate, 
							   const CString& pszCaption) 
				: CPropertyPage(nIDTemplate, 0)
{
	ASSERT_VALID(pSheet);
	m_pSheet = pSheet;
	m_strCaption = pszCaption;
	m_psp.pszTitle = m_strCaption;
	m_psp.dwFlags |= PSP_USETITLE;
	ASSERT_VALID(this);
}

void CMyPropertyPageEx::SetSheet(CPropertySheet* pSheet)
{
	ASSERT_VALID(this);
	ASSERT_VALID(pSheet);
	m_pSheet = pSheet;
}

CMyPropertyPageEx::~CMyPropertyPageEx()
{
}

void CMyPropertyPageEx::SetTabText(const CString& pszText)
{
	ASSERT_VALID(this);
	m_strCaption = pszText;
	ASSERT_VALID(m_pSheet);
	if(::IsWindow(m_pSheet->m_hWnd))
	{
		CTabCtrl* pTab = m_pSheet->GetTabControl();
		ASSERT(pTab);

		TC_ITEM Item;
		memset(&Item, 0, sizeof Item);
		Item.mask = TCIF_TEXT;
		(LPCTSTR&)Item.pszText = m_strCaption;
		pTab->SetItem(m_pSheet->GetPageIndex(this), &Item);
		pTab->Invalidate();
	}
	else
	{
		m_psp.pszTitle = m_strCaption;
		m_psp.dwFlags |= PSP_USETITLE;
	}
}

void CMyPropertyPageEx::SetTabIcon(const HICON hIcon)
{
	ASSERT_VALID(this);
	ASSERT_VALID(m_pSheet);
	if(::IsWindow(m_pSheet->m_hWnd))
	{
		// get tab control from parent sheet
		CTabCtrl* pTab = m_pSheet->GetTabControl();
		ASSERT(pTab);
		// get image list if present
		HIMAGELIST hImList;
		hImList = (HIMAGELIST)pTab->SendMessage(TCM_GETIMAGELIST, 0, 0L);
		CImageList ImList;
		if(hImList != NULL)
			ImList.Attach(hImList);
		else
			ImList.Create(::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON), TRUE, 4, 4);
		// get index of current icon
		TC_ITEM Item;
		Item.mask = TCIF_IMAGE;
		pTab->GetItem(m_pSheet->GetPageIndex(this), &Item);
		if(Item.iImage == -1)
		// add the icon
		{
			Item.iImage = ImList.Add(hIcon);
			pTab->SetItem(m_pSheet->GetPageIndex(this), &Item);
		}
		else
		// replace it
		{
			ImList.Replace(Item.iImage, hIcon);
		}
		if(hImList == NULL)
			pTab->SetImageList(&ImList);
		ImList.Detach();
		pTab->Invalidate();
	}
	else
	{
		m_psp.hIcon = hIcon;
		m_psp.dwFlags |= PSP_USEHICON;
		m_psp.dwFlags &= ~PSP_USEICONID;
	}
}

void CMyPropertyPageEx::SetTabIcon(const UINT resourceId)
{
	ASSERT_VALID(this);
	ASSERT_VALID(m_pSheet);
	HICON hIcon;
	hIcon = ::LoadIcon(AfxGetInstanceHandle(), MAKEINTRESOURCE(resourceId));
	VERIFY(NULL!=hIcon);
	SetTabIcon(hIcon);
}

void CMyPropertyPageEx::DoDataExchange(CDataExchange* pDX)
{
	ASSERT_VALID(this);
	CPropertyPage::DoDataExchange(pDX);
}

/////////////////////////////////////////////////////////////////////////////
// CMyPropertyPageEx message handlers

BOOL CMyPropertyPageEx::OnInitDialog() 
{
	ASSERT_VALID(this);
	CPropertyPage::OnInitDialog();
	return TRUE;  
}
