//*
//* cubematrix.cpp
//*
//* | Licensed Materials - Use restricted, please refer to the "Samples" terms
//* | and conditions in the IBM International Program License Agreement (IPLA).  
//* (C) Copyright IBM Corporation. 1992, 2006. All Rights Reserved.
//*

//
// CubeMatrix.cpp : implementation file

#include "stdafx.h"
#include "FasterActX.h"		// for class CMyFasterActXApp
#include "CubeMatrix.h"		// for class CMyCubeMatrix
#include "PageCubeMatrix.h"	// for class CMyPageCubeMatrix (parent notification)

#define DRAWCUBE	1		// display list number
#define WORKLOOPCOUNT	20

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMyCubeMatrix

CMyCubeMatrix::CMyCubeMatrix(UINT pt, BOOL job, BOOL loop)
{
    m_nAngle = 0;
	m_hPalette = NULL;
	m_nTimer = 0;
	m_loop = loop;
	m_fastwork = job;
	m_paintsDone = 0;
	m_rgbVal = 255;
	m_glfModifier0 = 0.03f;
	m_glfModifier2 = -0.02f;
	m_glfMaterialColor[0] = 0.0f;
	m_glfMaterialColor[1] = 0.0f;
	m_glfMaterialColor[2] = 1.0f;
	m_glfMaterialColor[3] = 1.0f;

	m_paints = ( pt <= 0 ) ? 250 : pt;
	m_elapse = 5;
}

CMyCubeMatrix::~CMyCubeMatrix()
{
	OnDestroy();
}


BEGIN_MESSAGE_MAP(CMyCubeMatrix, CWnd)
	//{{AFX_MSG_MAP(CMyCubeMatrix)
	ON_WM_PAINT()
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMyCubeMatrix message handlers

void CMyCubeMatrix::OnPaint() 
{
	if (m_nTimer &&  ++m_paintsDone > m_paints ) {
		KillTimer (m_nTimer);
		m_nTimer = 0;
		((CMyPageCubeMatrix*)(GetParent()->GetParent()))->OnStop();
		return;
	}

	CPaintDC dc(this); // device context for painting

	//
	// Clear the color and depth buffers.
	//
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//
	// Define the modelview transformation.
	//
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
	glTranslatef (0.0f, 0.0f, -32.0f);
	glRotatef (30.0f, 1.0f, 0.0f, 0.0f);
	glRotatef ((GLfloat) m_nAngle, 0.0f, 1.0f, 0.0f);

	if (m_loop) {
		Paint3DLoop();
	} else {
		Paint3DRecursive(-3, -3, -3);
	}

	//
	// Swap the buffers.
	//
	SwapBuffers (dc.m_hDC);

	// Do not call CWnd::OnPaint() for painting messages
}

void CMyCubeMatrix::Paint3DRecursive(int i, int j, int k) 
{
	if (m_fastwork) {
		RepetitiveJob1();
	} else {
		RepetitiveJob2();
	}

	//
	// Draw a three-dimensional array of cubes.
	//
	glPushMatrix ();
	glTranslatef ((GLfloat) i, (GLfloat) j, (GLfloat) k);
	glCallList (DRAWCUBE);
	glPopMatrix ();

	if ( k != 3) {
		Paint3DRecursive(i,   j,   k+3);
	} else if ( j != 3 ) {
		k = -3;
		Paint3DRecursive(i,   j+3, k);
	} else if (i != 3) {
		j = k = -3;
		Paint3DRecursive(i+3, j,   k);
	}
}

void CMyCubeMatrix::Paint3DLoop() 
{
	//
	// Draw a three-dimensional array of cubes.
	//
	int i, j, k;

	for (i=-3; i<=3; i+=3) {
		for (j=-3; j<=3; j+=3) {
			for (k=-3; k<=3; k+=3) {
				if (m_fastwork) {
					RepetitiveJob1();
				} else {
					RepetitiveJob2();
				}
				glPushMatrix ();
				glTranslatef ((GLfloat) i, (GLfloat) j, (GLfloat) k);
				glCallList (DRAWCUBE);
				glPopMatrix ();
			}
		}
	}
}


void CMyCubeMatrix::RepetitiveJob1()
{
	char buf[WORKLOOPCOUNT][20];

	for (int cnt = 0; cnt < WORKLOOPCOUNT; cnt++) {
		wsprintf(buf[cnt], "Hello World %d", cnt);
	}
}


void CMyCubeMatrix::RepetitiveJob2() 
{
	CString buf[WORKLOOPCOUNT];

	for (int cnt = 0; cnt < WORKLOOPCOUNT; cnt++) {
		buf[cnt].Format("Hello World %d", cnt);
	}
}


void CMyCubeMatrix::OnTimer(UINT nIDEvent) 
{
	//
	// Update the rotation angle and force a repaint.
	//
	if (m_nTimer == nIDEvent) {
		m_nAngle += 2;
			
		if (m_nAngle >= 360) {
			ModifyGLf();
			m_nAngle = 0;
		}
		InvalidateRect(NULL, FALSE);
	}
	
	CWnd::OnTimer(nIDEvent);
}

void CMyCubeMatrix::OnDestroy() 
{
    //
    // Clean up
    //
	glDeleteLists (DRAWCUBE, 1);
	wglMakeCurrent (NULL, NULL);
	wglDeleteContext (m_hrc);

	if (m_hWnd)
		ReleaseDC (m_pcdc);
	
	if (m_hPalette != NULL)
		DeleteObject (m_hPalette);

	if (m_nTimer) 
	{
		KillTimer (m_nTimer);
		m_nTimer = 0;
	}

	CWnd::OnDestroy();
}

BOOL CMyCubeMatrix::OnQueryNewPalette()
{
    //
    // If the program is using a color palette, realize the palette
    // and update the client area when the window receives the input
    // focus.
    //
	int n;
    if (m_hPalette != NULL) {
		n = m_pcdc->RealizePalette ();
        if (n)
            InvalidateRect (NULL, FALSE);
        return n;
    }
	return CWnd::OnQueryNewPalette();
}

void CMyCubeMatrix::OnPaletteChanged( CWnd* pFocusWnd )
{
    //
    // If the program is using a color palette, realize the palette
    // and update the colors in the client area when another program
    // realizes its palette.
    //
    if ((m_hPalette != NULL) && ( pFocusWnd != this)) {
        if ( m_pcdc->RealizePalette ())
            m_pcdc->UpdateColors ();
        return;
    }
	CWnd::OnPaletteChanged( pFocusWnd );
}

////////////////////////////////////////////////////////////////////////////
/*
 *  SetDCPixelFormat sets the pixel format for a device context in
 *  preparation for creating a rendering context.
 *
 */
 
void CMyCubeMatrix::SetDCPixelFormat (void)
{
    HANDLE hHeap;
    int nColors, i;
    LPLOGPALETTE lpPalette;
    BYTE byRedMask, byGreenMask, byBlueMask;

   	static PIXELFORMATDESCRIPTOR pfd = {
        sizeof (PIXELFORMATDESCRIPTOR),             // Size of this structure
        1,                                          // Version number
        PFD_DRAW_TO_WINDOW |                        // Flags
        PFD_SUPPORT_OPENGL |
        PFD_DOUBLEBUFFER,
        PFD_TYPE_RGBA,                              // RGBA pixel values
        24,                                         // 24-bit color
        0, 0, 0, 0, 0, 0,                           // Don't care about these
        0, 0,                                       // No alpha buffer
        0, 0, 0, 0, 0,                              // No accumulation buffer
        32,                                         // 32-bit depth buffer
        0,                                          // No stencil buffer
        0,                                          // No auxiliary buffers
        PFD_MAIN_PLANE,                             // Layer type
        0,                                          // Reserved (must be 0)
        0, 0, 0                                     // No layer masks
    };

    int nPixelFormat;
    
    nPixelFormat = ChoosePixelFormat (m_pcdc->m_hDC, &pfd);
    SetPixelFormat (m_pcdc->m_hDC, nPixelFormat, &pfd);

    DescribePixelFormat (m_pcdc->m_hDC, nPixelFormat, sizeof (PIXELFORMATDESCRIPTOR),
        &pfd);

    if (pfd.dwFlags & PFD_NEED_PALETTE) {
        nColors = 1 << pfd.cColorBits;
        hHeap = GetProcessHeap ();

        (LPLOGPALETTE) lpPalette = (LPLOGPALETTE)HeapAlloc (hHeap, 0,
            sizeof (LOGPALETTE) + (nColors * sizeof (PALETTEENTRY)));
            
        lpPalette->palVersion = 0x300;
        lpPalette->palNumEntries = (unsigned short) nColors;

        byRedMask = (BYTE) ((1 << pfd.cRedBits) - 1);
        byGreenMask = (BYTE) ((1 << pfd.cGreenBits) - 1);
        byBlueMask = (BYTE) ((1 << pfd.cBlueBits) - 1);

        for (i=0; i<nColors; i++) {
            lpPalette->palPalEntry[i].peRed =
                (BYTE) ((((i >> pfd.cRedShift) & byRedMask) * m_rgbVal) / byRedMask);
            lpPalette->palPalEntry[i].peGreen =
                (BYTE) ((((i >> pfd.cGreenShift) & byGreenMask) * m_rgbVal) / byGreenMask);
            lpPalette->palPalEntry[i].peBlue =
                (BYTE) ((((i >> pfd.cBlueShift) & byBlueMask) * m_rgbVal) / byBlueMask);
            lpPalette->palPalEntry[i].peFlags = 0;
        }

        m_hPalette = CreatePalette (lpPalette);
        HeapFree (hHeap, 0, lpPalette);

        if (m_hPalette != NULL) {
            SelectPalette (m_pcdc->m_hDC, m_hPalette, FALSE);
            RealizePalette (m_pcdc->m_hDC);
        }
    }
}

/*
 *  InitializeRC initializes the current rendering context.
 *
 *  Input parameters:
 *    None
 *
 *  Returns:
 *    Nothing
 */

void CMyCubeMatrix::InitializeRC (void)
{
    GLfloat glfLightAmbient[] = { 0.1f, 0.1f, 0.1f, 1.0f };
    GLfloat glfLightDiffuse[] = { 0.7f, 0.7f, 0.7f, 1.0f };
    GLfloat glfLightSpecular[] = { 0.0f, 0.0f, 0.0f, 1.0f };

    //
    // Enable depth testing and backface culling.
    //
    glEnable (GL_DEPTH_TEST);
    glEnable (GL_CULL_FACE);

    //
    // Add a light to the scene.
    //
    glLightfv (GL_LIGHT0, GL_AMBIENT, glfLightAmbient);
    glLightfv (GL_LIGHT0, GL_DIFFUSE, glfLightDiffuse);
    glLightfv (GL_LIGHT0, GL_SPECULAR, glfLightSpecular);
    glEnable (GL_LIGHTING);
    glEnable (GL_LIGHT0);
}

/*
 *  CreateDisplayList creates a display list that draws a cube.
 *
 *  Input parameters:
 *    None
 *
 *  Returns:
 *    Nothing
 */
 
void CMyCubeMatrix::CreateDisplayList (void)
{
    glNewList (DRAWCUBE, GL_COMPILE);
        glMaterialfv (GL_FRONT, GL_AMBIENT_AND_DIFFUSE, m_glfMaterialColor);

        glBegin (GL_POLYGON);
            glNormal3f (0.0f, 0.0f, 1.0f);
            glVertex3f (1.0f, 1.0f, 1.0f);
            glVertex3f (-1.0f, 1.0f, 1.0f);            
            glVertex3f (-1.0f, -1.0f, 1.0f);           
            glVertex3f (1.0f, -1.0f, 1.0f);            
        glEnd ();

        glBegin (GL_POLYGON);
            glNormal3f (0.0f, 0.0f, -1.0f);
            glVertex3f (1.0f, 1.0f, -1.0f);
            glVertex3f (1.0f, -1.0f, -1.0f);           
            glVertex3f (-1.0f, -1.0f, -1.0f);          
            glVertex3f (-1.0f, 1.0f, -1.0f);           
        glEnd ();

        glBegin (GL_POLYGON);
            glNormal3f (-1.0f, 0.0f, 0.0f);
            glVertex3f (-1.0f, 1.0f, 1.0f);
            glVertex3f (-1.0f, 1.0f, -1.0f);           
            glVertex3f (-1.0f, -1.0f, -1.0f);          
            glVertex3f (-1.0f, -1.0f, 1.0f);           
        glEnd ();

        glBegin (GL_POLYGON);
            glNormal3f (1.0f, 0.0f, 0.0f);
            glVertex3f (1.0f, 1.0f, 1.0f);
            glVertex3f (1.0f, -1.0f, 1.0f);            
            glVertex3f (1.0f, -1.0f, -1.0f);           
            glVertex3f (1.0f, 1.0f, -1.0f);            
        glEnd ();

        glBegin (GL_POLYGON);
            glNormal3f (0.0f, 1.0f, 0.0f);
            glVertex3f (-1.0f, 1.0f, -1.0f);
            glVertex3f (-1.0f, 1.0f, 1.0f);            
            glVertex3f (1.0f, 1.0f, 1.0f);             
            glVertex3f (1.0f, 1.0f, -1.0f);            
        glEnd ();

        glBegin (GL_POLYGON);
            glNormal3f (0.0f, -1.0f, 0.0f);
            glVertex3f (-1.0f, -1.0f, -1.0f);
            glVertex3f (1.0f, -1.0f, -1.0f);           
            glVertex3f (1.0f, -1.0f, 1.0f);            
            glVertex3f (-1.0f, -1.0f, 1.0f);           
        glEnd ();
    glEndList ();
}

//***************************************************************************

int CMyCubeMatrix::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	//
    // Create a rendering context and set a timer.
    //
	m_pcdc = GetDC ();
	SetDCPixelFormat ();
	m_hrc = wglCreateContext (m_pcdc->m_hDC);
	wglMakeCurrent (m_pcdc->m_hDC, m_hrc);
	InitializeRC ();
	CreateDisplayList ();
	m_nTimer = SetTimer (1, m_elapse, NULL);
	return 0;
}

void CMyCubeMatrix::SetRepetitiveJob(BOOL job)
{
	m_fastwork = job;
}


void CMyCubeMatrix::SetPaintMethod(BOOL meth)
{
	m_loop = meth;
}

UINT CMyCubeMatrix::SetPaintCount(UINT x)
{
	m_paints = (x <= 0) ? 250 : x;
	return m_paints;
}

UINT CMyCubeMatrix::ChangeTimer(UINT x)
{
	m_elapse = (x <= 0) ? 50 : x;
	KillTimer( m_nTimer );
	m_nTimer = SetTimer (1, m_elapse, NULL);
	return m_elapse;
}

void CMyCubeMatrix::OnSize(UINT nType, int cx, int cy) 
{
	UNREFERENCED_PARAMETER(nType);
	UNREFERENCED_PARAMETER(cx);
	UNREFERENCED_PARAMETER(cy);
    // Redefine the viewing volume and viewport when the window size
    // changes.
    //
    GLsizei glnWidth, glnHeight;
    GLdouble gldAspect;
	CRect cubeRect;
	GetClientRect(&cubeRect);

    glnWidth = (GLsizei) cubeRect.Width();
    glnHeight = (GLsizei) cubeRect.Height();
    gldAspect = (GLdouble) glnWidth / (GLdouble) glnHeight;

    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    gluPerspective (30.0,           // Field-of-view angle
                    gldAspect,      // Aspect ratio of viewing volume
                    1.0,            // Distance to near clipping plane
                    100.0);         // Distance to far clipping plane

    glViewport (0, 0, glnWidth, glnHeight);
}

void CMyCubeMatrix::ModifyGLf() 
{
	m_glfMaterialColor[0] += m_glfModifier0;
	m_glfMaterialColor[2] += m_glfModifier2;

	if ((m_glfMaterialColor[0] <= 0.0f) || (m_glfMaterialColor[0] >= 1.0f))
		m_glfModifier0 *= -1.0f;

	if ((m_glfMaterialColor[2] <= 0.0f) || (m_glfMaterialColor[2] >= 1.0f))
		m_glfModifier2 *= -1.0f;
			
	CreateDisplayList ();
}

void CMyCubeMatrix::OnLButtonDown(UINT nFlags, CPoint point) 
{
	UNREFERENCED_PARAMETER(nFlags);
	UNREFERENCED_PARAMETER(point);

	KillTimer (m_nTimer);
	m_nTimer = 0;
	((CMyPageCubeMatrix*)(GetParent()->GetParent()))->OnStop();
}

