////////////////////////////////////////////////////////////////////////////////
// FasterActX ActiveX Demonstration Program For Quantify
//
// | Licensed Materials - Use restricted, please refer to the "Samples" terms
// | and conditions in the IBM International Program License Agreement (IPLA).  
// (C) Copyright IBM Corporation. 1992, 2006. All Rights Reserved.
//
// Authors: Alan S. Cooper, Sean Echevarria
////////////////////////////////////////////////////////////////////////////////
// Typical Disclaimer: This FAQ is copyright (c) 1997 by Tom Armstrong
// (tom@widgetware.com). You have the right to copy and distribute this file 
// in any way you see fit as long as this paragraph is included with the 
// distribution. No warranties or claims are made as to the validity of the 
// information contained herein. So use it at your own risk.
////////////////////////////////////////////////////////////////////////////////
//
// PageCubeMatrix.cpp : implementation file

#include "stdafx.h"
#include "PageCubeMatrix.h"	// for class CMyPageCubeMatrix

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//----------------------------------------------------------------------------
// CMyPageCubeMatrix property page

BEGIN_MESSAGE_MAP(CMyPageCubeMatrix, CMyPropertyPageEx)
	//{{AFX_MSG_MAP(CMyPageCubeMatrix)
	ON_BN_CLICKED(IDC_RUN, OnRun)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CMyPageCubeMatrix::CMyPageCubeMatrix(void)
	: CMyPropertyPageEx(CMyPageCubeMatrix::IDD)
{
	//{{AFX_DATA_INIT(CMyPageCubeMatrix)
	m_paints = 75;
	m_slow = FALSE;
	m_InstructBox = _T("");
	//}}AFX_DATA_INIT
	m_CubeMatrix = NULL;
	m_isRunning = FALSE;
	ASSERT_VALID(this);
}

CMyPageCubeMatrix::~CMyPageCubeMatrix(void)
{
	m_InstructBox.Empty();
	if (m_CubeMatrix) {
		delete m_CubeMatrix;
	}
}

void CMyPageCubeMatrix::DoDataExchange(CDataExchange* pDX)
{
	ASSERT_VALID(this);
	CMyPropertyPageEx::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMyPageCubeMatrix)
	DDX_Control(pDX, IDC_RUN, m_runButton);
	DDX_Control(pDX, IDC_JOBBOX, m_slowBox);
	DDX_Control(pDX, IDC_EDIT1, m_paintEdit);
	DDX_Control(pDX, IDC_CUBEAREA, m_demoWnd);
	DDX_Text(pDX, IDC_EDIT1, m_paints);
	DDX_Check(pDX, IDC_JOBBOX, m_slow);
	DDX_Text(pDX, IDC_INSTRUCTIONS, m_InstructBox);
	//}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////

#ifdef _DEBUG
void CMyPageCubeMatrix::AssertValid() const
{
}

void CMyPageCubeMatrix::Dump(CDumpContext& dc) const
{
	UNREFERENCED_PARAMETER(dc);
}
#endif //_DEBUG

//----------------------------------------------------------------------------
// CMyPageCubeMatrix message handlers

BOOL CMyPageCubeMatrix::OnInitDialog() 
{
	CMyPropertyPageEx::OnInitDialog();

	m_InstructBox.LoadString(IDS_INSTRUCT_EXPENSIVE);
	UpdateData(FALSE);
	m_demoWnd.GetClientRect(&m_rect);
	m_font.CreatePointFont(90, "");
	GetDlgItem(IDC_INSTRUCTIONS)->SetFont(&m_font, FALSE);
	return TRUE;
}

void CMyPageCubeMatrix::OnRun()
{
	if (!UpdateData(TRUE))
		return;

	if (m_isRunning) {
		OnStop();
		return;
	}

	SetControls(FALSE);

	if (m_CubeMatrix) {
		delete m_CubeMatrix;
		m_CubeMatrix = NULL;
	}

	m_CubeMatrix = new CMyCubeMatrix(m_paints, m_slow);
	m_CubeMatrix->Create(NULL, "", WS_CHILD|WS_VISIBLE, m_rect, &m_demoWnd, IDC_CUBEAREA);
}

BOOL CMyPageCubeMatrix::OnSetActive() 
{
	ASSERT_VALID(this);
	SetControls(TRUE);

	// create the cube and let's paint it once so that we don't have an 
	// empty window on the first tab
	if (!m_CubeMatrix) {
		m_CubeMatrix = new CMyCubeMatrix(1, 1);
		m_CubeMatrix->Create(NULL, "", WS_CHILD|WS_VISIBLE, m_rect, &m_demoWnd, IDC_CUBEAREA);
	}

	return CMyPropertyPageEx::OnSetActive();
}

BOOL CMyPageCubeMatrix::OnKillActive() 
{
	OnStop();
	return CMyPropertyPageEx::OnKillActive();
}

void CMyPageCubeMatrix::OnStop() 
{
	if (m_CubeMatrix) {
		m_CubeMatrix->SetPaintCount(1);
	}

	SetControls(TRUE);
}

void CMyPageCubeMatrix::SetControls(BOOL set)
{
	m_paintEdit.EnableWindow(set);
	m_slowBox.EnableWindow(set);

	if (set) {
		m_runButton.SetWindowText(_T("Run"));
		m_isRunning = FALSE;
	} else {
		m_runButton.SetWindowText(_T("Stop"));
		m_isRunning = TRUE;
	}
}
