////////////////////////////////////////////////////////////////////////////////
// FasterActX ActiveX Demonstration Program For Quantify
//
// | Licensed Materials - Use restricted, please refer to the "Samples" terms
// | and conditions in the IBM International Program License Agreement (IPLA).  
// (C) Copyright IBM Corporation. 1992, 2006. All Rights Reserved.
//
// Authors: Alan S. Cooper, Sean Echevarria
////////////////////////////////////////////////////////////////////////////////
// Typical Disclaimer: This FAQ is copyright (c) 1997 by Tom Armstrong
// (tom@widgetware.com). You have the right to copy and distribute this file 
// in any way you see fit as long as this paragraph is included with the 
// distribution. No warranties or claims are made as to the validity of the 
// information contained herein. So use it at your own risk.
////////////////////////////////////////////////////////////////////////////////
//
// PageCubeThread.cpp : implementation file

#include "stdafx.h"
#include "PageCubeThread.h"	// for class CMyPageCubeThread

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//----------------------------------------------------------------------------
// CMyPageCubeThread property page

BEGIN_MESSAGE_MAP(CMyPageCubeThread, CMyPropertyPageEx)
	//{{AFX_MSG_MAP(CMyPageCubeThread)
	ON_BN_CLICKED(IDC_RUN, OnRun)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CMyPageCubeThread::CMyPageCubeThread(void)
	: CMyPropertyPageEx(CMyPageCubeThread::IDD)
{
	//{{AFX_DATA_INIT(CMyPageCubeThread)
	m_option = FALSE;
	//}}AFX_DATA_INIT
	m_SpinCube = NULL;
	m_isRunning = FALSE;
	ASSERT_VALID(this);
}

CMyPageCubeThread::~CMyPageCubeThread(void)
{
	m_InstructBox.Empty();
	m_help.Empty();
	if (m_SpinCube) {
		m_SpinCube->OnDestroy();
		delete m_SpinCube;
	}
}

void CMyPageCubeThread::DoDataExchange(CDataExchange* pDX)
{
	ASSERT_VALID(this);
	CMyPropertyPageEx::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMyPageCubeThread)
	DDX_Control(pDX, IDC_OPTIONBOX, m_optionBox);
	DDX_Control(pDX, IDC_RUN, m_runButton);
	DDX_Control(pDX, IDC_CUBEAREA, m_demoWnd);
	DDX_Check(pDX, IDC_OPTIONBOX, m_option);
	DDX_Text(pDX, IDC_INSTRUCTIONS, m_InstructBox);
	//}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////

#ifdef _DEBUG
void CMyPageCubeThread::AssertValid() const
{
}

void CMyPageCubeThread::Dump(CDumpContext& dc) const
{
	UNREFERENCED_PARAMETER(dc);
}
#endif //_DEBUG

//----------------------------------------------------------------------------
// CMyPageCubeThread message handlers

BOOL CMyPageCubeThread::OnInitDialog() 
{
	CMyPropertyPageEx::OnInitDialog();

	m_InstructBox.LoadString(IDS_INSTRUCT_THREADS);
	UpdateData(FALSE);
	m_demoWnd.GetClientRect(&m_rect);
	m_help.LoadString(IDS_ZOOMHELP);
	m_font.CreatePointFont(90, "");
	GetDlgItem(IDC_INSTRUCTIONS)->SetFont(&m_font, FALSE);
	return TRUE;
}

void CMyPageCubeThread::OnRun()
{
	UpdateData(TRUE);

	if (m_isRunning) {
		OnStop();
		return;
	}

	if (m_SpinCube) {
		m_SpinCube->OnDestroy();
		delete m_SpinCube;
		m_SpinCube = NULL;
	}

	m_optionBox.EnableWindow(FALSE);
	m_SpinCube = new CMySpinCube(!m_option);
	m_SpinCube->Create(NULL, "", WS_CHILD|WS_VISIBLE, m_rect, &m_demoWnd, IDC_CUBEAREA);
	m_runButton.SetWindowText(_T("Stop"));
	SetDlgItemText(IDC_HELPBOX, m_help);
	m_isRunning = TRUE;
}

BOOL CMyPageCubeThread::OnSetActive() 
{
	ASSERT_VALID(this);

	m_runButton.SetWindowText(_T("Run"));
	m_optionBox.EnableWindow(TRUE);
	SetDlgItemText(IDC_HELPBOX, "");
	return CMyPropertyPageEx::OnSetActive();
}

BOOL CMyPageCubeThread::OnKillActive() 
{
	OnStop();
	if (m_SpinCube) {
		m_SpinCube->OnDestroy();
		delete m_SpinCube;
		m_SpinCube = NULL;
	}

	return CMyPropertyPageEx::OnKillActive();
}

void CMyPageCubeThread::OnStop() 
{
	if (m_SpinCube)
	{
		m_SpinCube->StopDrawing();
	}

	m_optionBox.EnableWindow(TRUE);
	SetDlgItemText(IDC_HELPBOX, "");
	m_runButton.SetWindowText(_T("Run"));
	m_isRunning = FALSE;
}
