////////////////////////////////////////////////////////////////////////////////
// FasterActX ActiveX Demonstration Program For Quantify
//
// | Licensed Materials - Use restricted, please refer to the "Samples" terms
// | and conditions in the IBM International Program License Agreement (IPLA).  
// (C) Copyright IBM Corporation. 1992, 2006. All Rights Reserved.
//
// Authors: Alan S. Cooper, Sean Echevarria
////////////////////////////////////////////////////////////////////////////////
// Typical Disclaimer: This FAQ is copyright (c) 1997 by Tom Armstrong
// (tom@widgetware.com). You have the right to copy and distribute this file 
// in any way you see fit as long as this paragraph is included with the 
// distribution. No warranties or claims are made as to the validity of the 
// information contained herein. So use it at your own risk.
////////////////////////////////////////////////////////////////////////////////
//
// SpinCube.cpp : implementation file
//

#include "stdafx.h"
#include "fasteractx.h"
#include "SpinCube.h"
#include "gl\glaux.h"
#include "PageCubeThread.h"		// for class CMyPageCubeThread (parent notification)


#define THREAD_WAIT INFINITE


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
//

unsigned char threeto8[8] = 
{
	0, 0111>>1, 0222>>1, 0333>>1, 0444>>1, 0555>>1, 0666>>1, 0377
};

unsigned char twoto8[4] = 
{
	0, 0x55, 0xaa, 0xff
};

unsigned char oneto8[2] = 
{
	0, 255
};

static int defaultOverride[13] = 
{
	0, 3, 24, 27, 64, 67, 88, 173, 181, 236, 247, 164, 91
};

static PALETTEENTRY defaultPalEntry[20] = 
{
	{ 0,   0,   0,    0 },
	{ 0x80,0,   0,    0 },
	{ 0,   0x80,0,    0 },
	{ 0x80,0x80,0,    0 },
	{ 0,   0,   0x80, 0 },
	{ 0x80,0,   0x80, 0 },
	{ 0,   0x80,0x80, 0 },
	{ 0xC0,0xC0,0xC0, 0 },

	{ 192, 220, 192,  0 },
	{ 166, 202, 240,  0 },
	{ 255, 251, 240,  0 },
	{ 160, 160, 164,  0 },

	{ 0x80,0x80,0x80, 0 },
	{ 0xFF,0,   0,    0 },
	{ 0,   0xFF,0,    0 },
	{ 0xFF,0xFF,0,    0 },
	{ 0,   0,   0xFF, 0 },
	{ 0xFF,0,   0xFF, 0 },
	{ 0,   0xFF,0xFF, 0 },
	{ 0xFF,0xFF,0xFF, 0 }
};


/////////////////////////////////////////////////////////////////////////////
// CMySpinCube

CMySpinCube::CMySpinCube(BOOL sync)
{
	m_bActive = FALSE; 
	m_bMouseCaptured = FALSE; 
	m_pPal = NULL;
	m_hThreads[0] = m_hThreads[1] = NULL;
	m_threadIDs[0] = m_threadIDs[1] = 0;
	m_hMutex = NULL;
	m_sync = sync;
	m_useUnLock = TRUE;
	m_setNewContext = TRUE;
}

CMySpinCube::~CMySpinCube()
{
	OnDestroy();

	if (m_pPal)
	{
		delete[] m_pPal;
		m_pPal = NULL;
	}
}


BEGIN_MESSAGE_MAP(CMySpinCube, CWnd)
	//{{AFX_MSG_MAP(CMySpinCube)
	ON_WM_SIZE()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_DESTROY()
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMySpinCube methods

void CMySpinCube::CreateContext(CDC *cdc, HGLRC& glrc)
{
    PIXELFORMATDESCRIPTOR pfd;
	GLfloat		fMaxObjSize, fAspect;
	GLfloat		fNearPlane, fFarPlane;

    if (!bSetupPixelFormat(cdc->m_hDC))
        return;

    CreateRGBPalette(cdc);
	cdc->SelectPalette(&m_palette, FALSE);
	cdc->RealizePalette();

    int n = GetPixelFormat(cdc->m_hDC);
	DescribePixelFormat(cdc->m_hDC, n, sizeof(pfd), &pfd);

    glrc = wglCreateContext(cdc->m_hDC);
    wglMakeCurrent(cdc->m_hDC, glrc);

    glClearDepth(10.0f);
    glEnable(GL_DEPTH_TEST);

	if (m_hrcTxt.bottom)
		fAspect = (GLfloat)m_hrcTxt.right/m_hrcTxt.bottom;
	else	// don't divide by zero, not that we should ever run into that...
		fAspect = 1.0f;

	fNearPlane = 3.0f;
	fFarPlane = 20.0f;
	fMaxObjSize = 3.0f;
	m_fRadius = fNearPlane + fMaxObjSize / 2.0f;

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(30.0f, fAspect, fNearPlane, fFarPlane);
    glMatrixMode(GL_MODELVIEW);
}


unsigned char CMySpinCube::ComponentFromIndex(int i, UINT nbits, UINT shift)
{
    unsigned char val;

    val = (unsigned char) (i >> shift);
    switch (nbits) 
	{

    case 1:
        val &= 0x1;
        return oneto8[val];
    case 2:
        val &= 0x3;
        return twoto8[val];
    case 3:
        val &= 0x7;
        return threeto8[val];

    default:
        return 0;
    }
}


void CMySpinCube::CreateRGBPalette(CDC *dc)
{
    PIXELFORMATDESCRIPTOR pfd;
    int n, i;

	if (m_pPal)
		return;

    n = GetPixelFormat(dc->m_hDC);
    DescribePixelFormat(dc->m_hDC, n, sizeof(pfd), &pfd);

    if (pfd.dwFlags & PFD_NEED_PALETTE)
    {
        n = 1 << pfd.cColorBits;
        m_pPal = (PLOGPALETTE) new char[sizeof(LOGPALETTE) + n * sizeof(PALETTEENTRY)];

        _ASSERTE(m_pPal != NULL);

        m_pPal->palVersion = 0x300;
        m_pPal->palNumEntries = n;
        for (i=0; i<n; i++)
        {
            m_pPal->palPalEntry[i].peRed =
                    ComponentFromIndex(i, pfd.cRedBits, pfd.cRedShift);
            m_pPal->palPalEntry[i].peGreen =
                    ComponentFromIndex(i, pfd.cGreenBits, pfd.cGreenShift);
            m_pPal->palPalEntry[i].peBlue =
                    ComponentFromIndex(i, pfd.cBlueBits, pfd.cBlueShift);
            m_pPal->palPalEntry[i].peFlags = 0;
        }

        /* fix up the palette to include the default GDI palette */
        if ((pfd.cColorBits == 8)                           &&
            (pfd.cRedBits   == 3) && (pfd.cRedShift   == 0) &&
            (pfd.cGreenBits == 3) && (pfd.cGreenShift == 3) &&
            (pfd.cBlueBits  == 2) && (pfd.cBlueShift  == 6)
           )
        {
			for (i = 1 ; i <= 12 ; i++)
                m_pPal->palPalEntry[defaultOverride[i]] = defaultPalEntry[i];
        }

		m_palette.CreatePalette((LPLOGPALETTE)m_pPal);

        dc->SelectPalette(&m_palette, FALSE);
        dc->RealizePalette();
    }
}


BOOL CMySpinCube::bSetupPixelFormat(HDC hdc)
{
    static PIXELFORMATDESCRIPTOR pfd = 
	{
        sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
        1,                              // version number
        PFD_DRAW_TO_WINDOW |            // support window
          PFD_SUPPORT_OPENGL |          // support OpenGL
          PFD_DOUBLEBUFFER,             // double buffered
        PFD_TYPE_RGBA,                  // RGBA type
        24,                             // 24-bit color depth
        0, 0, 0, 0, 0, 0,               // color bits ignored
        0,                              // no alpha buffer
        0,                              // shift bit ignored
        0,                              // no accumulation buffer
        0, 0, 0, 0,                     // accum bits ignored
        32,                             // 32-bit z-buffer
        0,                              // no stencil buffer
        0,                              // no auxiliary buffer
        PFD_MAIN_PLANE,                 // main layer
        0,                              // reserved
        0, 0, 0                         // layer masks ignored
    };
    int pixelformat;

    if ( (pixelformat = ChoosePixelFormat(hdc, &pfd)) == 0 )
    {
        return FALSE;
    }

    if (SetPixelFormat(hdc, pixelformat, &pfd) == FALSE)
    {
        return FALSE;
    }

    return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CMySpinCube message handlers

void CMySpinCube::OnSize(UINT nType, int cx, int cy) 
{
	UNREFERENCED_PARAMETER(nType);
	UNREFERENCED_PARAMETER(cx);
	UNREFERENCED_PARAMETER(cy);
	
	GetClientRect(&m_hrcTxt);
	m_setNewContext = TRUE;
}

void CMySpinCube::OnMouseMove(UINT nFlags, CPoint point) 
{
	UNREFERENCED_PARAMETER(nFlags);
	UNREFERENCED_PARAMETER(point);

	if (m_bMouseCaptured)
	{
		GLfloat fNearPlane = 3.0f;
		GLfloat fFarPlane = 7.0f;
		GLfloat fMaxObjSize = 3.0f;
		m_fRadius += (float)(m_xPos - point.x)/100.0f;
		m_xPos = point.x;
		m_yPos = point.x;
	}
}

void CMySpinCube::OnLButtonUp(UINT nFlags, CPoint point) 
{
	UNREFERENCED_PARAMETER(nFlags);
	UNREFERENCED_PARAMETER(point);

	m_bMouseCaptured = FALSE;
	ReleaseCapture();
}

void CMySpinCube::OnLButtonDown(UINT nFlags, CPoint point) 
{
	UNREFERENCED_PARAMETER(nFlags);

	SetCapture();
	m_bMouseCaptured = TRUE;
	m_xPos = point.x;  // horizontal position of cursor 
	m_yPos = point.y;  // vertical position of cursor 
}

void CMySpinCube::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	UNREFERENCED_PARAMETER(nFlags);
	UNREFERENCED_PARAMETER(point);

	m_bActive = FALSE;
	((CMyPageCubeThread*)(GetParent()->GetParent()))->OnStop();
}

void CMySpinCube::StopDrawing()
{
	m_bActive = FALSE;
}

void CMySpinCube::OnDestroy() 
{
	m_bActive = FALSE;
	WaitForMultipleObjects(2, m_hThreads, TRUE, ((THREAD_WAIT == INFINITE) ? 1000 : THREAD_WAIT) * 2);

	if (m_hMutex) {
		CloseHandle(m_hMutex);
		m_hMutex = NULL;
	}

	if (m_hThreads[0]) {
		CloseHandle(m_hThreads[0]);
		CloseHandle(m_hThreads[1]);
		m_hThreads[0] = m_hThreads[1] = NULL;
	}

	CWnd::OnDestroy();
}

int CMySpinCube::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1) {
		return -1;
	}
	
	if (m_sync) {
		m_hMutex = CreateMutex(NULL, FALSE, NULL);
	}

	m_bActive = TRUE;
	m_hThreads[0] = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)DrawOurCubeAndGrid, 
		this, 0, &m_threadIDs[0]);
	m_hThreads[1] = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)DrawOurText,
		this, 0, &m_threadIDs[1]);
	
	return 0;
}

void CMySpinCube::DrawOurText(CMySpinCube *thisPtr)
{
	CDC		*dc = thisPtr->GetDC();
	DWORD	waitSuccess = 1;
	CString captionGood, captionBad, time;
	SYSTEMTIME currSysT, startSysT, diffSysT;
	
	GetSystemTime(&startSysT);

	for (DWORD passes = 0; thisPtr->m_bActive; passes++) {
		if (thisPtr->m_sync) {
			waitSuccess = WaitForSingleObject(thisPtr->m_hMutex, THREAD_WAIT);
		}

		GetSystemTime(&currSysT);

		if (startSysT.wMilliseconds <= currSysT.wMilliseconds) {
			diffSysT.wMilliseconds = currSysT.wMilliseconds - startSysT.wMilliseconds;
		} else {
			currSysT.wSecond-=1;
			diffSysT.wMilliseconds = currSysT.wMilliseconds - startSysT.wMilliseconds + 1000;
		}

		if (startSysT.wSecond <= currSysT.wSecond && currSysT.wSecond < 60) {
			diffSysT.wSecond = currSysT.wSecond - startSysT.wSecond;
		} else {
			currSysT.wMinute-=1;
			diffSysT.wSecond = currSysT.wSecond - startSysT.wSecond + 60;
		}

		if (startSysT.wMinute <= currSysT.wMinute && currSysT.wMinute < 60) {
			diffSysT.wMinute = currSysT.wMinute - startSysT.wMinute;
		} else {
			// no borrowing minutes - if you run this for more than an hour, the elapsed time will be wrong
			diffSysT.wMinute = currSysT.wMinute - startSysT.wMinute + 60;
		}

		time.Format("%d min %02d.%04d sec", diffSysT.wMinute, diffSysT.wSecond, 
			diffSysT.wMilliseconds);

		captionGood.Format("Thread_0x%x pass %d (%s)", thisPtr->m_threadIDs[1], passes, time);
		captionBad.Format("Thread_0x%x timedout (%s)", thisPtr->m_threadIDs[1], time);

		dc->SelectPalette(&thisPtr->m_palette, FALSE);
		dc->RealizePalette();
		dc->SetBkMode(OPAQUE);
		dc->SetTextColor(RGB(0,0,0));
		dc->DrawText(((waitSuccess == WAIT_TIMEOUT) ? captionBad : captionGood), -1, &thisPtr->m_hrcTxt,
			DT_CENTER|DT_BOTTOM| DT_SINGLELINE);

		if (thisPtr->m_sync) {
			ReleaseMutex(thisPtr->m_hMutex);
		}
	}

	// print out the last caption again since the other thread may have overwritten it
	WaitForSingleObject(thisPtr->m_hThreads[0], ((THREAD_WAIT == INFINITE) ? 1000 : THREAD_WAIT) * 2);

	dc->SelectPalette(&thisPtr->m_palette, FALSE);
	dc->RealizePalette();
	dc->SetBkMode(OPAQUE);
	dc->SetTextColor(RGB(0,0,0));
	dc->DrawText(((waitSuccess == WAIT_TIMEOUT) ? captionBad : captionGood), -1, &thisPtr->m_hrcTxt,
		DT_CENTER|DT_BOTTOM| DT_SINGLELINE);

	captionGood.Empty();
	captionBad.Empty();
	time.Empty();
}

void CMySpinCube::DrawOurCubeAndGrid(CMySpinCube *thisPtr)
{
	CDC		*dc = thisPtr->GetDC();
	HGLRC	hGlrc = NULL;
	GLfloat	wAngleY = 10.0f;
	GLfloat	wAngleX = 1.0f;
	GLfloat	wAngleZ = 5.0f;
	CString caption;

	caption.Format("Thread_0x%x accidentally blocked", thisPtr->m_threadIDs[1]);

	while (thisPtr->m_bActive) {
		if (thisPtr->m_sync) {
			WaitForSingleObject(thisPtr->m_hMutex, THREAD_WAIT);
		}

		if (thisPtr->m_setNewContext) {
			thisPtr->m_setNewContext = FALSE;
			wglMakeCurrent(NULL,  NULL);
			if (hGlrc)
			{
				wglDeleteContext(hGlrc);
				hGlrc = NULL;
			}
			thisPtr->CreateContext(dc, hGlrc);
		}

		glClearColor(0.0f, 0.0f, 0.0f, 10.0f);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		glPushMatrix();
			glTranslatef(0.0f, 0.0f, -thisPtr->m_fRadius);
			glRotatef(wAngleX, 1.0f, 0.0f, 0.0f);
			glRotatef(wAngleY, 0.0f, 1.0f, 0.0f);
			glRotatef(wAngleZ, 0.0f, 0.0f, 1.0f);

			wAngleX -= (float(0x3000 - 32768) / 32768.0f) * 20.0f;
			wAngleY -= (float(0x4000 - 32768) / 32768.0f) * 20.0f;
			wAngleZ += 1.0f;

			glBegin(GL_QUAD_STRIP);
				glColor3f(1.0f, 0.0f, 1.0f);
				glVertex3f(-0.5f, 0.5f, 0.5f);

				glColor3f(1.0f, 0.0f, 0.0f);
				glVertex3f(-0.5f, -0.5f, 0.5f);

				glColor3f(1.0f, 1.0f, 1.0f);
				glVertex3f(0.5f, 0.5f, 0.5f);

				glColor3f(1.0f, 1.0f, 0.0f);
				glVertex3f(0.5f, -0.5f, 0.5f);

				glColor3f(0.0f, 1.0f, 1.0f);
				glVertex3f(0.5f, 0.5f, -0.5f);

				glColor3f(0.0f, 1.0f, 0.0f);
				glVertex3f(0.5f, -0.5f, -0.5f);
			
				glColor3f(0.0f, 0.0f, 1.0f);
				glVertex3f(-0.5f, 0.5f, -0.5f);

				glColor3f(0.0f, 0.0f, 0.0f);
				glVertex3f(-0.5f, -0.5f,  -0.5f);

				glColor3f(1.0f, 0.0f, 1.0f);
				glVertex3f(-0.5f, 0.5f, 0.5f);

				glColor3f(1.0f, 0.0f, 0.0f);
				glVertex3f(-0.5f, -0.5f, 0.5f);
			glEnd();

			glBegin(GL_QUADS);
				glColor3f(1.0f, 0.0f, 1.0f);
				glVertex3f(-0.5f, 0.5f, 0.5f);

				glColor3f(1.0f, 1.0f, 1.0f);
				glVertex3f(0.5f, 0.5f, 0.5f);

				glColor3f(0.0f, 1.0f, 1.0f);
				glVertex3f(0.5f, 0.5f, -0.5f);

				glColor3f(0.0f, 0.0f, 1.0f);
				glVertex3f(-0.5f, 0.5f, -0.5f);
			glEnd();

			glBegin(GL_QUADS);
				glColor3f(1.0f, 0.0f, 0.0f);
				glVertex3f(-0.5f, -0.5f, 0.5f);

				glColor3f(1.0f, 1.0f, 0.0f);
				glVertex3f(0.5f, -0.5f, 0.5f);

				glColor3f(0.0f, 1.0f, 0.0f);
				glVertex3f(0.5f, -0.5f, -0.5f);

				glColor3f(0.0f, 0.0f, 0.0f);
				glVertex3f(-0.5f, -0.5f,  -0.5f);
			glEnd();

			glColor3f(1.0f, 0.0f, 0.0f);
//			auxWireIcosahedron(2.5);
//			auxWireCube(2.5);
//			auxWireSphere(2.5);
//			auxWireOctahedron(2.5);
//			auxWireTetrahedron(2.5);
//			auxWireDodecahedron(2.5);
//			auxWireTorus(2.5, 1.0);
			auxWireTeapot(3.0);		// hey! this works on multiple consecutive instantiations
//			auxWireBox(2.5, 1.0, 1.5);
//			auxWireCylinder(2.5, 1.0);
//			auxWireCone(2.5, 1.0);
			
		glPopMatrix();

		glFinish();
		SwapBuffers(wglGetCurrentDC());
		
		if (thisPtr->m_useUnLock) {
			if (thisPtr->m_sync) {
				ReleaseMutex(thisPtr->m_hMutex);
			}
		} else {
			// Oops, forgot that I called WaitForSingleObject() at the top
			dc->SetBkMode(OPAQUE);
			dc->SetTextColor(RGB(0,0,0));
			dc->DrawText(caption, -1, &thisPtr->m_hrcTxt, DT_CENTER|DT_BOTTOM|DT_SINGLELINE);
		}
	}

    wglMakeCurrent(NULL,  NULL);
    wglDeleteContext(hGlrc);
	caption.Empty();
}
