//*
//* vqdemocon1.h
//*
//* Licensed Materials - Use restricted, please refer to the "Samples" terms
//* and conditions in the IBM International Program License Agreement (IPLA).  
//* (C) Copyright IBM Corporation. 2002, 2006. All Rights Reserved.
//*

// VQDemoCon1.h : Declaration of the CVQDemoCon

#ifndef __VQDEMOCON_H_
#define __VQDEMOCON_H_

#include "resource.h"       // main symbols
#include <oledlg.h>

/////////////////////////////////////////////////////////////////////////////
// CVQDemoCon
class CVQDemoCon : 
	public CComObjectRoot,
	public CComCoClass<CVQDemoCon, &CLSID_CVQDemoCon>,
	public CWindowImpl<CVQDemoCon>,
	public IVQDemoCon,
	public IOleClientSite,
	public IOleInPlaceSite
{
public:
	CVQDemoCon()
	{
		m_bInPlaceActive = FALSE;
		m_pOleObject = NULL;
	}

DECLARE_REGISTRY_RESOURCEID(IDR_VQDemoCon)

BEGIN_COM_MAP(CVQDemoCon)
	COM_INTERFACE_ENTRY(IVQDemoCon)
	COM_INTERFACE_ENTRY(IOleClientSite)
	COM_INTERFACE_ENTRY(IOleWindow)
	COM_INTERFACE_ENTRY(IOleInPlaceSite)
END_COM_MAP()

BEGIN_MSG_MAP(CVQDemoCon)
	MESSAGE_HANDLER(WM_NCDESTROY, OnNCDestroy)
	MESSAGE_HANDLER(WM_SIZE, OnSize)
	MESSAGE_HANDLER(WM_ERASEBKGND, OnErase)
END_MSG_MAP()

	LRESULT OnNCDestroy(UINT /* uMsg */, WPARAM /* wParam */, LPARAM /* lParam */, BOOL& /* lResult */)
	{
		Stop();
		return 0;
	}

	LRESULT OnErase(UINT /* uMsg */, WPARAM /* wParam */, LPARAM /* lParam */, BOOL& bHandled)
	{
		if (m_bInPlaceActive)
			return 0;
		bHandled = FALSE;
		return 1;
	}

	LRESULT OnSize(UINT /* uMsg */, WPARAM /* wParam */, LPARAM /* lParam */, BOOL& /* lResult */)
	{
		if (m_pOleObject && m_bInPlaceActive)
		{
			CComQIPtr<IOleInPlaceObject, &IID_IOleInPlaceObject> pInPlaceObject(m_pOleObject);
			RECT rcClient;
			GetClientRect(&rcClient);
			pInPlaceObject->SetObjectRects(&rcClient, &rcClient);
		}
		return 0;
	}

	LRESULT OpenControl()
	{
		MSG			msg;
		RECT		rcPos;
		CLSID		clsid;
		HINSTANCE	module;
		FARPROC		regDll;

		m_bInPlaceActive = FALSE;

		// forcibly load the ocx and ask it to self-register so that we can 
		// get a CLSID in the next statement
		if (!(module = LoadLibrary("fasteractx.ocx"))) 
		{
			MessageBox(_T("VQDemoCon was unable to load fasteractx.ocx."),
				_T("LoadLibrary Error"));
			return 0;
		}
		if (!(regDll = GetProcAddress(module, _T("DllRegisterServer")))) 
		{
			DWORD err = GetLastError();
			char msg[_MAX_PATH];
			wsprintf(msg, "VQDemoCon was unable to call DllRegisterServer for fasteractx.ocx.\nError number %d", err);
			MessageBox(msg,	_T("GetProcAddress Error"));
			return 0;
		}
		(*regDll)();

		// ProgID = s 'FasterActXATL.FasterActXATL.1' from FasterActXATL.rgs
		CLSIDFromProgID(OLESTR("FasterActXATL.FasterActXATL.1"), &clsid);
		
		CoCreateInstance(clsid, NULL, CLSCTX_INPROC, IID_IOleObject, (void**)&m_pOleObject);
		if (!m_pOleObject)
		{
			MessageBox(_T("CoCreateInstance failed.\nPlease build the sample ActiveX control before running this program."),
				_T("ActiveX Control Error"));
			return 0;
		}
		m_pOleObject->SetClientSite(this);

		GetWindowRect(&rcPos);
		m_pOleObject->DoVerb(OLEIVERB_INPLACEACTIVATE, &msg, this, 0, m_hWnd, &rcPos);

		// resize window after creating the object - the values here are for 
		// "small font" display settings.  If using "large font" display settings,
		// then the user will have resize the window manually.
		GetWindowRect(&rcPos);
		//#define FASTWIDTH		472		FasterActX control width from FasterActXATL.cpp
		//#define FASTHEIGHT	407		FasterActX control height from FasterActXATL.cpp
		MoveWindow(rcPos.left, rcPos.top, 472 + 2, 407 + 22, TRUE);

		ShowWindow(SW_SHOWNORMAL);

		return 1;
	}

// IVQDemoCon
public:
	STDMETHOD(Run)()
	{
		RECT rcPos = { CW_USEDEFAULT, 0, 0, 0 };
		Create(GetDesktopWindow(), rcPos, _T("Quantify Demo Container"), WS_VISIBLE | WS_OVERLAPPEDWINDOW, NULL);
		if (!OpenControl()) 
		{
			PostQuitMessage(0);
		} 
		return S_OK;
	}
	STDMETHOD(Stop)()
	{
		if (m_pOleObject) {
			m_pOleObject->Close(OLECLOSE_NOSAVE);
			m_pOleObject = NULL;
		}

		PostQuitMessage(0);
		return S_OK;
	}

// IOleClientSite
public:
    STDMETHOD(SaveObject)(void)
	{
		ATLTRACENOTIMPL(_T("IOleClientSite::SaveObject"));
	}
    STDMETHOD(GetMoniker)(DWORD dwAssign, DWORD dwWhichMoniker, IMoniker **ppmk)
	{
		ATLTRACENOTIMPL(_T("IOleClientSite::GetMoniker"));
	}
    STDMETHOD(GetContainer)(IOleContainer **ppContainer)
	{
		ATLTRACENOTIMPL(_T("IOleClientSite::GetContainer"));
	}
    STDMETHOD(ShowObject)(void)
	{
		ATLTRACENOTIMPL(_T("IOleClientSite::ShowObject"));
	}
    STDMETHOD(OnShowWindow)(BOOL fShow)
	{
		ATLTRACENOTIMPL(_T("IOleClientSite::OnShowWindow"));
	}
    STDMETHOD(RequestNewObjectLayout)(void)
	{
		ATLTRACENOTIMPL(_T("IOleClientSite::RequestNewObjectLayout"));
	}


// IOleWindow
public:
    STDMETHOD(GetWindow)(HWND *phwnd)
	{
		*phwnd = m_hWnd;
		return S_OK;
	}
    STDMETHOD(ContextSensitiveHelp)(BOOL fEnterMode)
	{
		ATLTRACENOTIMPL(_T("IOleWindow::CanInPlaceActivate"));
	}

// IOleInPlaceSite
public:
    STDMETHOD(CanInPlaceActivate)(void)
	{
		return S_OK;
	}
    STDMETHOD(OnInPlaceActivate)(void)
	{
		m_bInPlaceActive = TRUE;
		return S_OK;
	}
    STDMETHOD(OnUIActivate)(void)
	{
		ATLTRACENOTIMPL(_T("IOleInPlaceSite::OnUIActivate"));
	}
    STDMETHOD(GetWindowContext)(IOleInPlaceFrame **ppFrame, IOleInPlaceUIWindow **ppDoc,
		LPRECT lprcPosRect, LPRECT lprcClipRect, LPOLEINPLACEFRAMEINFO lpFrameInfo)
	{
		GetClientRect(lprcPosRect);
		GetClientRect(lprcClipRect);
		return S_OK;
	}
    STDMETHOD(Scroll)(SIZE scrollExtant)
	{
		ATLTRACENOTIMPL(_T("IOleInPlaceSite::Scroll"));
	}
    STDMETHOD(OnUIDeactivate)(BOOL fUndoable)
	{
		ATLTRACENOTIMPL(_T("IOleInPlaceSite::OnUIDeactivate"));
	}
    STDMETHOD(OnInPlaceDeactivate)( void)
	{
		m_bInPlaceActive = FALSE;
		return S_OK;
	}
    STDMETHOD(DiscardUndoState)( void)
	{
		ATLTRACENOTIMPL(_T("IOleInPlaceSite::DiscardUndoState"));
	}
    STDMETHOD(DeactivateAndUndo)( void)
	{
		ATLTRACENOTIMPL(_T("IOleInPlaceSite::DeactivateAndUndo"));
	}
    STDMETHOD(OnPosRectChange)(LPCRECT lprcPosRect)
	{
		ATLTRACENOTIMPL(_T("IOleInPlaceSite::OnPosRectChange"));
	}

	CComPtr<IOleObject> m_pOleObject;
	BOOL m_bInPlaceActive;
};

#endif //__VQDEMOCON_H_
